mod conn;
pub mod entities;
mod migrator;
mod query;

use sea_orm::{ConnectOptions, Database, DatabaseConnection, DeriveActiveEnum, EnumIter};
use sea_orm_migration::MigratorTrait;
use serde::{Deserialize, Serialize};

pub use entities::{prelude::*, *};
use migrator::Migrator;

type Result<T> = std::result::Result<T, sea_orm::DbErr>;

#[derive(EnumIter, DeriveActiveEnum, Serialize, Deserialize, PartialEq, Eq, Clone, Debug)]
#[sea_orm(rs_type = "i32", db_type = "Integer")]
pub enum PackageRelatedEnum {
    #[sea_orm(num_value = 0)]
    Conflicts,
    #[sea_orm(num_value = 1)]
    Replaces,
    #[sea_orm(num_value = 2)]
    Provides,
    #[sea_orm(num_value = 3)]
    Depend,
    #[sea_orm(num_value = 4)]
    Makedepend,
    #[sea_orm(num_value = 5)]
    Checkdepend,
    #[sea_orm(num_value = 6)]
    Optdepend,
}

#[derive(Serialize)]
pub struct FullPackage {
    #[serde(flatten)]
    entry: package::Model,
    licenses: Vec<String>,
    groups: Vec<String>,
    related: Vec<(PackageRelatedEnum, String)>,
    files: Vec<String>,
}

#[derive(Clone, Debug)]
pub struct RieterDb {
    conn: DatabaseConnection,
    pub pkg: query::PackageQuery,
    pub repo: query::RepoQuery,
}

impl RieterDb {
    pub async fn connect<C: Into<ConnectOptions>>(opt: C) -> Result<Self> {
        let db = Database::connect(opt).await?;

        Migrator::up(&db, None).await?;

        Ok(Self {
            conn: db.clone(),
            pkg: query::PackageQuery::new(db.clone()),
            repo: query::RepoQuery::new(db.clone()),
        })
    }
}
